#!/bin/csh -f

# This is the file we use to make the serialized grammars.  It won't work
# for you, since it contains hard-coded paths to various treebanks.  But
# it may be useful to inspect it to see what options we used to generate
# the various grammars.

set wsjptb=/afs/ir/data/linguistic-data/Treebank/3/parsed/mrg/wsj
set ctb3=/afs/ir/data/linguistic-data/Chinese-Treebank/3/bracketed
set negra=/afs/ir/data/linguistic-data/NEGRA/penn-format-train-dev-test
set host=`hostname | cut -d. -f1`

# English PCFG binary and text grammars

( echo $host server ; time java -server -ms300m -mx1000m edu.stanford.nlp.parser.lexparser.LexicalizedParser -goodPCFG -saveToSerializedFile englishPCFG.ser.gz -saveToTextFile englishPCFG.txt -train $wsjptb 200 2199 ) >& /tmp/serializedParsers.log

# English noTagSplit PCFG text
( echo $host server ; time java -server -ms300m -mx1000m edu.stanford.nlp.parser.lexparser.LexicalizedParser -goodPCFG -noTagSplit -train $wsjptb 200 2199 -saveToAscii englishPCFG-noTagSplit.txt -compactGrammar 0 ) >>& /tmp/serializedParsers.log

# English Factored binary

( echo $host server ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -ijcai03 -train $wsjptb 200 2199 englishFactored.ser.gz ) >>& /tmp/serializedParsers.log

# Chinese PCFG binary

( echo $host server ; time java -server -mx800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -tLPP edu.stanford.nlp.parser.lexparser.ChineseTreebankParserParams -PCFG -chinesePCFG -encoding GB18030 -train $ctb3 026 931 chinesePCFG.ser.gz ) >>& /tmp/serializedParsers.log

# Chinese Factored binary

( echo $host server ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -tLPP edu.stanford.nlp.parser.lexparser.ChineseTreebankParserParams -factored -encoding GB18030 -acl03chinese -train $ctb3 026 931 chineseFactored.ser.gz ) >>& /tmp/serializedParsers.log

# German Factored binary

( echo $host server ; time java -server -ms200m -mx1800m edu.stanford.nlp.parser.lexparser.LexicalizedParser -tLPP edu.stanford.nlp.parser.lexparser.NegraPennTreebankParserParams -hMarkov 1 -train $negra 1 1  germanFactored.ser.gz ) >>& /tmp/serializedParsers.log
