package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.util.Numberer;

import java.io.Serializable;

/**
 * Binary rules (ints for parent, left and right children)
 *
 * @author Dan Klein
 */

public class BinaryRule extends Rule implements Serializable, Comparable {

  public int leftChild = -1;
  public int rightChild = -1;

  public BinaryRule() {
  }

  public BinaryRule(int parent, int leftChild, int rightChild, double score) {
    this.parent = parent;
    this.leftChild = leftChild;
    this.rightChild = rightChild;
    this.score = score;
  }

  public int hashCode() {
    return (parent << 16) ^ (leftChild << 8) ^ rightChild;
  }

  public boolean equals(Object o) {
    if (this == o)
      return true;
    if (o instanceof BinaryRule) {
      BinaryRule br = (BinaryRule) o;
      if (parent == br.parent &&
              leftChild == br.leftChild &&
              rightChild == br.rightChild)
        return true;
    }
    return false;
  }

  public String toString() {
    Numberer stateNum = Numberer.getGlobalNumberer("states");
    return stateNum.object(parent) + "\t->\t" + stateNum.object(leftChild) +
      "\t" + stateNum.object(rightChild) + "\t" + score;
  }

  public int compareTo(Object o) {
    BinaryRule ur = (BinaryRule) o;
    if (parent < ur.parent) return -1;
    if (parent > ur.parent) return 1;
    if (leftChild < ur.leftChild) return -1;
    if (leftChild > ur.leftChild) return 1;
    if (rightChild < ur.rightChild) return -1;
    if (rightChild > ur.rightChild) return 1;
    return 0;
  }

  private static final long serialVersionUID = 1L;

}
