package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.trees.*;

import java.util.*;

/**
 */
public class CNFTransformers {
  static class ToCNFTransformer implements TreeTransformer {
    public Tree transformTree(Tree t, TreeFactory tf) {
      if (t.isLeaf())
        return t.treeFactory().newLeaf(t.label());
      Tree[] children = t.children();
      if (children.length > 1 || t.isPreTerminal() || t.label().value().startsWith("ROOT")) {
        Label label = t.label();
        Tree[] transformedChildren = new Tree[children.length];
        for (int childIndex = 0; childIndex < children.length; childIndex++) {
          Tree child = children[childIndex];
          transformedChildren[childIndex] = transformTree(child, tf);
        }
        return t.treeFactory().newTreeNode(label, Arrays.asList(transformedChildren));
      }
      Tree tree = t;
      List conjoinedList = new ArrayList();
      while (tree.children().length == 1 && ! tree.isPrePreTerminal()) {
        String nodeString = tree.label().value();
        if (! nodeString.startsWith("@"))
          conjoinedList.add(nodeString);
        tree = tree.children()[0];
      }
      String nodeString = tree.label().value();
      if (! nodeString.startsWith("@"))
        conjoinedList.add(nodeString);
      String conjoinedLabels = "";
      if (conjoinedList.size() > 1) {
        for (Iterator cI = conjoinedList.iterator(); cI.hasNext();) {
          String s = (String) cI.next();
          conjoinedLabels = conjoinedLabels + "&" + s;
        }
      } else if (conjoinedList.size() == 1) {
        conjoinedLabels = (String) conjoinedList.iterator().next();
      } else {
        return transformTree(t.children()[0], tf);
      }
      children = tree.children();
      Label label = t.label().labelFactory().newLabel(conjoinedLabels);
      Tree[] transformedChildren = new Tree[children.length];
      for (int childIndex = 0; childIndex < children.length; childIndex++) {
        Tree child = children[childIndex];
        transformedChildren[childIndex] = transformTree(child, tf);
      }
      return t.treeFactory().newTreeNode(label, Arrays.asList(transformedChildren));
    }
  }

  static class FromCNFTransformer implements TreeTransformer {
    public Tree transformTree(Tree t, TreeFactory tf) {
      if (t.isLeaf())
        return t.treeFactory().newLeaf(t.label());
      Tree[] children = t.children();
      Tree[] transformedChildren = new Tree[children.length];
      for (int childIndex = 0; childIndex < children.length; childIndex++) {
        Tree child = children[childIndex];
        transformedChildren[childIndex] = transformTree(child, tf);
      }
      Label label = t.label();
      if (! label.value().startsWith("&")) {
        return t.treeFactory().newTreeNode(label, Arrays.asList(transformedChildren));
      }
      String[] nodeStrings = label.value().split("&");
      int i = nodeStrings.length - 1;
      label = t.label().labelFactory().newLabel(nodeStrings[i]);
      Tree result = t.treeFactory().newTreeNode(label, Arrays.asList(transformedChildren));
      while (i > 1) {
        i--;
        label = t.label().labelFactory().newLabel(nodeStrings[i]);
        result = t.treeFactory().newTreeNode(label, Collections.singletonList(result));
      }
      return result;
    }
  }

  public static void main(String[] args) {
    CategoryWordTag.printWordTag = false;
    String path = args[0];
    List trees = TreebankAnnotator.getTrees(path, 200, 219, 0, 10);
    List annotatedTrees = new TreebankAnnotator().annotateTrees(trees);
    for (Iterator annotatedTreesI = annotatedTrees.iterator(); annotatedTreesI.hasNext();) {
      Tree tree = (Tree) annotatedTreesI.next();
      System.out.println("ORIGINAL:\n");
      tree.pennPrint();
      System.out.println("CNFed:\n");
      Tree cnfTree = new ToCNFTransformer().transformTree(tree, null);
      cnfTree.pennPrint();
      System.out.println("UnCNFed:\n");
      Tree unCNFTree = new FromCNFTransformer().transformTree(cnfTree, null);
      unCNFTree.pennPrint();
      System.out.println("\n\n");
    }
  }


}


