package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.trees.*;
import edu.stanford.nlp.util.*;
import java.util.*;
import java.io.*;


public class ConstituentEvalByCat extends AbstractEval {
  public ConstituentEvalByCat(String str) {
    this.str = str;
    lce = new LabeledConstituentEval(str);
  }

  public String str;

  LabeledConstituentEval lce;

  private static final Set emptySet = new HashSet();

  Map guessDeps = new HashMap();
  Map goldDeps = new HashMap();

  Counter precisions = new Counter();
  Counter recalls = new Counter();
  Counter f1s = new Counter();
  Counter nums = new Counter();

  Counter precisions2 = new Counter();
  Counter recalls2 = new Counter();
  Counter pnums2 = new Counter();
  Counter rnums2 = new Counter();

  double num = 0.0;

  /* we don't use this directly, it's here just to make it an AbstractEval*/
  Set makeObjects(Tree tree) {
    return lce.makeObjects(tree);
  }

  /* returns a map of categories to sets of constituents*/
  private Map makeObjectsByCat(Tree t) {
    Map objMap = new HashMap();
    Set objSet = makeObjects(t);
    for(Iterator i = objSet.iterator(); i.hasNext();){
      LabeledConstituent lc = (LabeledConstituent) i.next();
      Label l = lc.label();
      if(!objMap.keySet().contains(l))
	objMap.put(l,new HashSet());
      ((HashSet) objMap.get(l)).add(lc);
    }
    return objMap;
  }


  public void evaluate(Tree guess, Tree gold, PrintWriter pw) {
    guessDeps = makeObjectsByCat(guess);
    goldDeps = makeObjectsByCat(gold);

    Set cats = new HashSet();
    cats.addAll(guessDeps.keySet());
    cats.addAll(goldDeps.keySet());

    pw.println("Labeled Bracketed Evaluation by Category");
    pw.println("****************************************");

    num += 1.0;

    for(Iterator i = cats.iterator(); i.hasNext();){
      Label cat = (Label) i.next();
      Set thisGuessDeps = (Set) guessDeps.get(cat);
      Set thisGoldDeps = (Set) goldDeps.get(cat);

      if(thisGuessDeps == null)
	thisGuessDeps = emptySet;
      if(thisGoldDeps == null)
	thisGoldDeps = emptySet;

      double currentPrecision = precision(thisGuessDeps,thisGoldDeps);
      double currentRecall = precision(thisGoldDeps,thisGuessDeps);

      double currentF1 = (currentPrecision > 0.0 && currentRecall > 0.0 ? 2.0/(1.0/currentPrecision+1.0/currentRecall) : 0.0);

      precisions.incrementCount(cat,currentPrecision);
      recalls.incrementCount(cat,currentRecall);
      f1s.incrementCount(cat,currentF1);

      precisions2.incrementCount(cat,thisGuessDeps.size()*currentPrecision);
      pnums2.incrementCount(cat,thisGuessDeps.size());

      recalls2.incrementCount(cat,thisGoldDeps.size()*currentRecall);
      rnums2.incrementCount(cat,thisGoldDeps.size());


      pw.println(cat+"\tP: "+((int)(currentPrecision*10000))/100.0 +
		 " (sent ave "+((int)(precisions.getCount(cat)*10000/num))/100.0+") (evalb "+((int)(precisions2.getCount(cat)*10000/pnums2.getCount(cat)))/100.0+")");
      pw.println("\tR: "+((int)(currentRecall*10000))/100.0 +
		 " (sent ave "+((int)(recalls.getCount(cat)*10000/num))/100.0+") (evalb "+((int)(recalls2.getCount(cat)*10000/rnums2.getCount(cat)))/100.0+")");
      
      double cF1 = 2.0/(rnums2.getCount(cat)/(double)recalls2.getCount(cat)+pnums2.getCount(cat)/(double)precisions2.getCount(cat));
      String emit = str + " F1: " + ((int)(currentF1*10000))/100.0 + " (sent ave " +
        ((int)(10000*f1s.getCount(cat)/num))/100.0+", evalb "+((int)(10000*cF1))/100.0 + ")";
      pw.println(emit);
    }
    pw.println("****************************************");
    lce.evaluate(guess,gold,pw); // evaluate regular LabeledConsituentEval
  }

  public void display(boolean verbose, PrintWriter pw) {

    Set cats = new HashSet();
    cats.addAll(precisions.keySet());
    cats.addAll(recalls.keySet());

    pw.println("Labeled Bracketed Evaluation by Category -- final statistics");
    pw.println("****************************************");

    for(Iterator i = cats.iterator(); i.hasNext(); ){
      Object cat = i.next();
      double pnum2 = pnums2.getCount(cat);
      double rnum2 = rnums2.getCount(cat);
      double prec = precisions2.getCount(cat)/pnum2;//(num > 0.0 ? precision/num : 0.0);
      double rec = recalls2.getCount(cat)/rnum2;//(num > 0.0 ? recall/num : 0.0);
      double f = 2.0/(1.0/prec+1.0/rec);//(num > 0.0 ? f1/num : 0.0);

      pw.println(cat + "\tLP: "+((int)(10000.0*prec))/100.0+ "\ttotal guessed: " + pnum2 + 
		 "\tLR: "+((int)(10000.0*rec))/100.0+ "\ttotal gold: " + rnum2 +  "\tF1: "+((int)(10000.0*f))/100.0);
    }
    pw.println("****************************************");

    lce.display(verbose,pw);

  }

}
