package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.util.Numberer;
import edu.stanford.nlp.trees.TaggedWord;

import java.io.Serializable;

public class IntTaggedWord implements Serializable, Comparable {

  private static Numberer wordNumberer;
  private static Numberer tagNumberer;

  Numberer wordNumberer() {
    if (wordNumberer == null)
      wordNumberer = Numberer.getGlobalNumberer("words");
    return wordNumberer;
  }

  Numberer tagNumberer() {
    if (tagNumberer == null)
      tagNumberer = Numberer.getGlobalNumberer("tags");
    return tagNumberer;
  }

  public int word;
  public short tag;

  public static final String ANY = ".*.";
  public static final String STOP = "STOP";

  public int tag() {
    return tag;
  }

  public int word() {
    return word;
  }

  public int hashCode() {
    return word ^ (tag << 16);
  }

  public boolean equals(Object o) {
    if (this == o) {
      return true;
    } else if (o instanceof IntTaggedWord) {
      IntTaggedWord i = (IntTaggedWord) o;
      return (word == i.word && tag == i.tag);
    } else {
      return false;
    }
  }

  public int compareTo(Object obj) {
    IntTaggedWord that = (IntTaggedWord) obj;
    if (tag != that.tag) {
      return tag - that.tag;
    } else {
      return word - that.word;
    }
  }

  public String toString() {
    String wordStr;
    String tagStr;
    if (word >= 0) {
      wordStr = (String) wordNumberer().object(word);
    } else if (word == -1) {
      wordStr = ANY;
    } else {
      wordStr = STOP;
    }
    if (tag >= 0) {
      tagStr = (String) tagNumberer().object(tag);
    } else if (tag == -1) {
      tagStr = ANY;
    } else {
      tagStr = STOP;
    }
    return wordStr + "|" + tagStr;
  }

  public IntTaggedWord(int word, int tag) {
    this.word = word;
    this.tag = (short) tag;
  }

  public TaggedWord toTaggedWord() {
    String wordStr;
    String tagStr;
    if (word >= 0) {
      wordStr = (String) wordNumberer().object(word);
    } else if (word == -1) {
      wordStr = ANY;
    } else {
      wordStr = STOP;
    }
    if (tag >= 0) {
      tagStr = (String) tagNumberer().object(tag);
    } else if (tag == -1) {
      tagStr = ANY;
    } else {
      tagStr = STOP;
    }
    return new TaggedWord(wordStr, tagStr);
  }

  /**
   * Creates an IntTaggedWord given by the String representation
   * of the form <word>|<tag>
   */
  public IntTaggedWord(String s) throws Exception {
    String wordString, tagString;
    int word, tag;
    int n = s.indexOf('|');
    wordString = s.substring(0, n);
    tagString = s.substring(n + 1, s.length());
    if (wordString.equals(ANY)) {
      word = -1;
    } else if (wordString.equals(STOP)) {
      word = -2;
    } else {
      word = wordNumberer().number(wordString);
    }
    if (tagString.equals(ANY)) {
      tag = -1;
    } else if (wordString.equals(STOP)) {
      tag = -2;
    } else {
      tag = tagNumberer().number(tagString);
    }
    this.word = word;
    this.tag = (short) tag;
  }

  private static final long serialVersionUID = 1L;

} // end class IntTaggedWord
