package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.trees.*;
import edu.stanford.nlp.trees.international.*;
import edu.stanford.nlp.trees.international.negra.*;
import edu.stanford.nlp.util.StringUtils;

import java.io.*;
import java.util.*;


/**
 * Parameter file for parsing the Penn Treebank format of the Negra
 * Treebank (German).  STILL UNDER CONSTRUCTION!
 *
 * @author Roger Levy
 */

public class NegraPennTreebankParserParams extends AbstractTreebankParserParams implements TreebankLangParserParams {

  public static boolean markRC = false;
  public static boolean markZuVP = false;
  public static boolean treeNormalizerInsertNPinPP = false;


  private boolean treeNormalizerLeaveGF = false;

  private NegraPennLanguagePack nplp = (NegraPennLanguagePack) treebankLanguagePack();

  public NegraPennTreebankParserParams() {
    super(new NegraPennLanguagePack());
    setInputEncoding(tlp.getEncoding());
    setOutputEncoding(tlp.getEncoding());
  }


  /** returns a NegraHeadFinder */
  public HeadFinder headFinder() {
    //NegraHeadFinder hf = new NegraHeadFinder();

    return new NegraHeadFinder();
    //return new LeftHeadFinder();
  }

  /** returns an ordinary Lexicon (could be tuned for German!) */
  public Lexicon lex() {
    return new GermanLexicon();
  }
  
  /* Returns a MemoryTreebank with a NegraPennTokenizer and a
   * NegraPennTreeNormalizer */
  public MemoryTreebank memoryTreebank() {
    final NegraPennTreeNormalizer tn = new NegraPennTreeNormalizer();
    if( treeNormalizerLeaveGF )
      tn.setLeaveGF(true);
    if( treeNormalizerInsertNPinPP )
      tn.setInsertNPinPP(true);
    return new MemoryTreebank(new TreeReaderFactory() {
	public TreeReader newTreeReader(Reader in) {
	  return new PennTreeReader(in, 
				new LabeledScoredTreeFactory(
			        new StringLabelFactory()),
				tn,
				new NegraPennTokenizer(in));
	    }
      },
			      inputEncoding);
  }

  /** returns a NegraPennCollinizer */
  public TreeTransformer collinizer() {
    return new NegraPennCollinizer();
  }

  /** returns a NegraPennCollinizer */
  public TreeTransformer collinizerEvalb() {
    return new NegraPennCollinizer(false);
  }

  /** Returns a <code>ChineseTreebankLanguagePack</code> */
  public TreebankLanguagePack treebankLanguagePack() {
    return new NegraPennLanguagePack();
  }


  /* parser tuning follows */

  private int selectiveSplitLevel = 0;



  public String[] sisterSplitters() {
    return new String[0];
  }


  // Automatically generated by ParentAnnotationStats -- preferably don't edit
  private static String[] splitters1 = new String[] {"VA^VCD", "NP^NP", "NP^VP", "NP^IP", "NP^DNP", "NP^PP", "NP^LCP", "NP^PRN", "NP^QP", "PP^IP", "PP^NP", "NN^FRAG", "NN^NP", "NT^FRAG", "NT^NP", "NR^FRAG", "NR^NP", "VV^FRAG", "VV^VRD", "VV^VCD", "VV^VP", "VV^VSB", "VP^VP", "VP^IP", "VP^DVP", "IP^ROOT", "IP^IP", "IP^CP", "IP^VP", "IP^PP", "IP^NP", "IP^LCP", "CP^IP", "QP^NP", "QP^PP", "QP^VP", "ADVP^CP", "CC^VP", "CC^NP", "CC^IP", "CC^QP", "PU^NP", "PU^FRAG", "PU^IP", "PU^VP", "PU^PRN", "PU^QP", "PU^LST", "NP^DNP~QP", "NT^NP~NP", "NT^NP~VP", "NT^NP~IP", "NT^NP~LCP", "NT^NP~PP", "NT^NP~PRN", "NT^NP~QP", "NT^NP~DNP", "NP^NP~VP", "NP^NP~NP", "NP^NP~IP", "NP^NP~PP", "NP^NP~DNP", "NP^NP~LCP", "NN^NP~VP", "NN^NP~IP", "NN^NP~NP", "NN^NP~PP", "NN^NP~DNP", "NN^NP~LCP", "NN^NP~UCP", "NN^NP~QP", "NN^NP~PRN", "M^CLP~DP", "M^CLP~QP", "M^CLP~NP", "M^CLP~CLP", "CD^QP~VP", "CD^QP~NP", "CD^QP~QP", "CD^QP~LCP", "CD^QP~PP", "CD^QP~DNP", "CD^QP~DP", "CD^QP~IP", "IP^IP~IP", "IP^IP~ROOT", "IP^IP~VP", "LC^LCP~PP", "LC^LCP~IP", "NP^VP~IP", "NP^VP~VP", "AD^ADVP~IP", "AD^ADVP~QP", "AD^ADVP~VP", "AD^ADVP~NP", "AD^ADVP~PP", "AD^ADVP~ADVP", "NP^IP~ROOT", "NP^IP~IP", "NP^IP~CP", "NP^IP~VP", "DT^DP~PP", "P^PP~IP", "P^PP~NP", "P^PP~VP", "P^PP~DNP", "VV^VP~IP", "VV^VP~VP", "PU^IP~IP", "PU^IP~VP", "PU^IP~ROOT", "PU^IP~CP", "JJ^ADJP~DNP", "JJ^ADJP~ADJP", "NR^NP~IP", "NR^NP~NP", "NR^NP~PP", "NR^NP~VP", "NR^NP~DNP", "NR^NP~LCP", "NR^NP~PRN", "NP^PP~NP", "NP^PP~IP", "NP^PP~DNP", "VA^VP~VP", "VA^VP~IP", "VA^VP~DVP", "VP^VP~VP", "VP^VP~IP", "VP^VP~DVP", "VP^IP~ROOT", "VP^IP~CP", "VP^IP~IP", "VP^IP~VP", "VP^IP~PP", "VP^IP~LCP", "VP^IP~NP", "PN^NP~NP", "PN^NP~IP", "PN^NP~PP"};
  private static String[] splitters2 = new String[] {"VA^VCD", "NP^NP", "NP^VP", "NP^IP", "NP^DNP", "NP^PP", "NP^LCP", "NN^FRAG", "NN^NP", "NT^FRAG", "NT^NP", "NR^FRAG", "NR^NP", "VV^FRAG", "VV^VRD", "VV^VCD", "VV^VP", "VV^VSB", "VP^VP", "VP^IP", "VP^DVP", "IP^ROOT", "IP^IP", "IP^CP", "IP^VP", "IP^PP", "CP^IP", "ADVP^CP", "CC^VP", "CC^NP", "PU^NP", "PU^FRAG", "PU^IP", "PU^VP", "PU^PRN", "NT^NP~NP", "NT^NP~VP", "NT^NP~IP", "NT^NP~LCP", "NT^NP~PP", "NP^NP~VP", "NP^NP~NP", "NP^NP~IP", "NP^NP~PP", "NP^NP~DNP", "NN^NP~VP", "NN^NP~IP", "NN^NP~NP", "NN^NP~PP", "NN^NP~DNP", "NN^NP~LCP", "NN^NP~UCP", "NN^NP~QP", "NN^NP~PRN", "M^CLP~DP", "CD^QP~VP", "CD^QP~NP", "CD^QP~QP", "CD^QP~LCP", "CD^QP~PP", "CD^QP~DNP", "CD^QP~DP", "LC^LCP~PP", "NP^VP~IP", "NP^VP~VP", "AD^ADVP~IP", "AD^ADVP~QP", "AD^ADVP~VP", "AD^ADVP~NP", "NP^IP~ROOT", "NP^IP~IP", "NP^IP~CP", "NP^IP~VP", "P^PP~IP", "P^PP~NP", "P^PP~VP", "P^PP~DNP", "VV^VP~IP", "VV^VP~VP", "PU^IP~IP", "PU^IP~VP", "PU^IP~ROOT", "PU^IP~CP", "JJ^ADJP~DNP", "NR^NP~IP", "NR^NP~NP", "NR^NP~PP", "NR^NP~VP", "NR^NP~DNP", "NR^NP~LCP", "NP^PP~NP", "VA^VP~VP", "VA^VP~IP", "VP^VP~VP", "VP^IP~ROOT", "VP^IP~CP", "VP^IP~IP", "VP^IP~VP", "VP^IP~PP", "VP^IP~LCP", "VP^IP~NP", "PN^NP~NP"};
  private static String[] splitters3 = new String[] {"NP^NP", "NP^VP", "NP^IP", "NP^DNP", "NP^PP", "NP^LCP", "NN^FRAG", "NN^NP", "NT^FRAG", "NR^FRAG", "NR^NP", "VV^FRAG", "VV^VRD", "VV^VCD", "VV^VP", "VV^VSB", "VP^VP", "VP^IP", "IP^ROOT", "IP^IP", "IP^CP", "IP^VP", "PU^NP", "PU^FRAG", "PU^IP", "PU^VP", "PU^PRN", "NP^NP~VP", "NN^NP~VP", "NN^NP~IP", "NN^NP~NP", "NN^NP~PP", "NN^NP~DNP", "NN^NP~LCP", "M^CLP~DP", "CD^QP~VP", "CD^QP~NP", "CD^QP~QP", "AD^ADVP~IP", "AD^ADVP~QP", "AD^ADVP~VP", "P^PP~IP", "VV^VP~IP", "VV^VP~VP", "PU^IP~IP", "PU^IP~VP", "NR^NP~IP", "NR^NP~NP", "NR^NP~PP", "NR^NP~VP", "VP^VP~VP", "VP^IP~ROOT", "VP^IP~CP", "VP^IP~IP", "VP^IP~VP"};
  private static String[] splitters4 = new String[] {"NP^NP", "NP^VP", "NP^IP", "NN^FRAG", "NT^FRAG", "NR^FRAG", "VV^FRAG", "VV^VRD", "VV^VCD", "VP^VP", "VP^IP", "IP^ROOT", "IP^IP", "IP^CP", "IP^VP", "PU^NP", "PU^FRAG", "PU^IP", "PU^VP", "NN^NP~VP", "NN^NP~IP", "NN^NP~NP", "NN^NP~PP", "NN^NP~DNP", "NN^NP~LCP", "CD^QP~VP", "CD^QP~NP", "AD^ADVP~IP", "VV^VP~IP", "VV^VP~VP", "NR^NP~IP", "VP^IP~ROOT", "VP^IP~CP"};
  // these ones were built by hand.
  // one can't tag split under FRAG or everything breaks, because of those
  // big flat FRAGs....
  private static String[] splitters5 =
    new String[] {"NN^FRAG", "NT^FRAG", "NR^FRAG", "VV^FRAG", 
		  "VV^VCD", "VV^VRD",
		  "NP^NP", "VP^VP",
		  "IP^ROOT", "IP^IP",
		  "PU^NP", "PU^FRAG",
		  "P^PP~VP", "P^PP~IP"};
  private static String[] splitters6 =
    new String[] {"VV^VCD", "VV^VRD",
		  "NP^NP", "VP^VP",
		  "IP^ROOT", "IP^IP",
		  "PU^NP",
		  "P^PP~VP", "P^PP~IP"};
  private static String[] splitters7 =
    new String[] {"NP^NP", "VP^VP",
		  "IP^ROOT", "IP^IP",
		  "PU^NP",
		  "P^PP~VP", "P^PP~IP"};
  private static String[] splitters8 =
    new String[] {"IP^ROOT", "IP^IP",
		  "PU^NP",
		  "P^PP~VP", "P^PP~IP"};
  private static String[] splitters9 =
    new String[] {"VV^VCD", "VV^VRD",
		  "NP^NP", "VP^VP",
		  "IP^ROOT", "IP^IP",
		  "P^PP~VP", "P^PP~IP"};
  private static String[] splitters10 =
    new String[] {"NP^NP", "VP^VP",
		  "IP^ROOT", "IP^IP",
		  "P^PP~VP", "P^PP~IP"};


  public String[] splitters() {
    switch (selectiveSplitLevel) {
    case 1:
      return splitters1;
    case 2:
      return splitters2;
    case 3:
      return splitters3;
    case 4:
      return splitters4;
    case 5:
      return splitters5;
    case 6:
      return splitters6;
    case 7:
      return splitters7;
    case 8:
      return splitters8;
    case 9:
      return splitters9;
    case 10:
      return splitters10;
    default:
      return new String[0];
    }
  }


  public int setOptionFlag(String[] args, int i) {
    if (args[i].equalsIgnoreCase("-leaveGF")) {
      treeNormalizerLeaveGF = true;
      i+=1;
    }
    if (args[i].equalsIgnoreCase("-markZuVP")) {
      markZuVP = true;
      i+=1;
    }
    if (args[i].equalsIgnoreCase("-markRC")) {
      markRC = true;
      i+=1;
    }
    if (args[i].equalsIgnoreCase("-insertNPinPP")) {
      treeNormalizerInsertNPinPP = true;
      i+=1;
    }
    return i;
  }

  public void display() {
    System.out.println("markZuVP=" + markZuVP);
    System.out.println("insertNPinPP=" + treeNormalizerInsertNPinPP);
  }




  private String basicCat(String str) {
    return nplp.basicCategory(str);
  }

  /** transformTree does all language-specific tree
   *  transformations. Any parameterizations should be inside the
   *  specific TreebankLangParserarams class.
   */
  public TreeHeadPair transformTree(Tree t, Tree root, TreeHeadPair thp) {
    Set annotations = new HashSet();

    String cat = t.label().value();
    String word = thp.head.word();
    String tag = thp.head.tag();

    Tree parent = t.parent(root);




    String mcat = "";
    if(parent != null)
      mcat = parent.label().value();


    //categories
    if(! (t.isLeaf() || t.isPreTerminal())) {

      List childBasicCats = childBasicCats(t);


      // mark vp's headed by "zu" verbs
      if(markZuVP && basicCat(cat).equals("VP")) {
	System.out.println("child basic cats: " + childBasicCats);
      }
      if(markZuVP && basicCat(cat).equals("VP") && (childBasicCats.contains("VZ") || childBasicCats.contains("VVIZU"))) {
	System.out.println("Marked zu VP" + t);
	annotations.add("-ZU");
      }

      // mark relative clause S's
      if(markRC && (t.label() instanceof NegraLabel) && tlp.basicCategory(t.label().value()).equals("S") && ((NegraLabel) t.label()).getEdge() != null && ((NegraLabel) t.label()).getEdge().equals("RC")) {
//	 ) {
	System.out.println("annotating this guy as RC:");
	t.pennPrint();
	//throw new RuntimeException("damn, not a Negra Label");
	
	annotations.add("-RC");
      }



    }
    


    // put on all the annotations
    for(Iterator i = annotations.iterator(); i.hasNext();){
      cat += (String) i.next();
    }

    t.setLabel(new CategoryWordTag(cat, word, tag));
    

    thp.tree = t;
    //thp.head = head;
    
    return thp;
  }

  private List childBasicCats(Tree t) {
    Tree[] kids = t.children();
    List l = new ArrayList();
    for(int i = 0, n = kids.length; i < n; i++) {
      l.add(basicCat(kids[i].label().value()));
    }
    return l;
  }


  private static long serialVersionUID = 2;

  public static void main(String[] args) {
    TreebankLangParserParams tlpp = new NegraPennTreebankParserParams();
    Treebank tb = tlpp.memoryTreebank();
    tb.loadPath(args[0]);
    for(Iterator i = tb.iterator(); i.hasNext();) {
      ((Tree) i.next()).pennPrint();
    }
  }    
					


}
