package edu.stanford.nlp.parser.lexparser;

import edu.stanford.nlp.util.*;
import edu.stanford.nlp.trees.*;
import edu.stanford.nlp.fsm.*;
import edu.stanford.nlp.io.NumberRangeFileFilter;

import java.util.*;
import java.io.Serializable;


/**
 * Extracts raw Nary rules from a treebank. They are returned as a Map from
 * passive constituents to Lists of right-hand side rule "paths", each of which is a List.
 */
class PathExtractor extends AbstractTreeExtractor {
  private static Object END = "END";
  protected Numberer stateNumberer = Numberer.getGlobalNumberer("states");
  private Map allPaths = new HashMap();
  private HeadFinder hf;

  public PathExtractor(HeadFinder hf) {
    this.hf = hf;
  }

  private List getList(Object key) {
    List result = (List) allPaths.get(key);
    if (result == null) {
      result = new ArrayList();
      allPaths.put(key, result);
    }
    return result;
  }

  protected void tallyInternalNode(Tree lt) {
    Tree[] children = lt.children();
    Tree headChild = hf.determineHead(lt);
    if (children.length == 1)
      return;
    List path = new ArrayList();

    // determine which is the head
    int headLoc = -1;
    for (int i = 0; i < children.length; i++) {
      if (children[i] == headChild) {
        headLoc = i;
      }
    }

    path.add(children[headLoc].label().value());
    if (headLoc == 0) {
      // we are finishing on the right
      for (int i = headLoc + 1; i < children.length - 1; i++)
        path.add(children[i].label().value() + ">");
      if (Train.markFinalStates) {
        path.add(children[children.length - 1].label().value() + "]");
      } else {
        path.add(children[children.length - 1].label().value() + ">");
      }
    } else {
      // we are finishing on the left
      for (int i = headLoc + 1; i < children.length; i++)
        path.add(children[i].label().value() + ">");
      for (int i = headLoc - 1; i > 0; i--)
        path.add(children[i].label().value() + "<");
      if (Train.markFinalStates) {
        path.add(children[0].label().value() + "[");
      } else {
        path.add(children[0].label().value() + "<");
      }
    }
    path.add(END); // add epsilon at the end
    String label = lt.label().value();
    List l = getList(label);
    l.add(path);
  }

  protected Object formResult() {
    return allPaths;
  }

}

