package edu.stanford.nlp.parser.lexparser;

/** Options to the parser which affect performance only at testing (parsing)
 *  time.
 */
public class Test {

  /** If false, then failure of the PCFG factor to parse a sentence
   * will trigger parse recovery mode, in which more flexible tagging
   * is allowed. False by default. Use option -noRecoveryTagging to set
   * to true.
   */
  public static boolean noRecoveryTagging = false;

  /* If true, then  failure of the PCFG factor to parse a sentence
   * will trigger parse recovery mode.
   */
  public static boolean doRecovery = true;

  /** If true, the n^4 "speed-up" is not used. */
  public static boolean useN5 = false;

  /** The maximum sentence length (including punctuation, etc.) to parse. */
  public static int maxLength = 40;

  /** Parse trees in test treebank in order of increasing length. */
  public static boolean increasingLength = false;

  /** Tag the sentences first, then parse given those (coarse) tags. */
  public static boolean preTag = false;

  /** Parse using only tags given from correct answer or the POS tagger */
  public static boolean forceTags = preTag;


  /** Write EvalB-readable output files. */
  public static boolean evalb = false;

  /** Print a lot of extra output as you parse. */
  public static boolean verbose = false; // Don't change this; set with -v

  public static final boolean exhaustiveTest = false;
  public static final boolean pcfgThreshold = false;
  public static final double pcfgThresholdValue = -2.0;

  /** If true, remove any node which improves the PCFG score. */
  public static boolean nodePrune = false;

  /** Print out all best PCFG parses. */
  public static boolean printAllBestParses = false;

  /** Weighting on dependency log probs */
  public static double depWeight = 1.0;
  public static boolean prunePunc = false;


  /** Determines format of output trees: choose among penn, oneline  */
  public static String outputTreeFormat = "penn";

  /** determines whether output trees have lexicalized categories displayed */
  public static boolean lexicalizeOutput = false;
  public static boolean headMarkOutput = false;
  public static boolean printDepsSet = false;

  /** Determines output-use headfinder for trees.  BADNESS HERE!!!
   *  There's no guarantee that this headfinder matches the one used
   *  during training XXX */

  public static TreePrint treePrint() {
    TreePrint tp = new TreePrint(outputTreeFormat);
    tp.setLexicalize(lexicalizeOutput);
    tp.setHeadMarkNodes(headMarkOutput);
    tp.setPrintDepsSet(printDepsSet);
    return tp;
  }

  public static void display() {
    String str = "Test parameters maxLength=" + maxLength + " nPrune=" + nodePrune + " preTag=" + preTag;
    System.err.println(str);
  }

}
