package edu.stanford.nlp.process;

import java.util.*;
import java.net.URL;
import java.io.*;

import edu.stanford.nlp.dbm.*;
import edu.stanford.nlp.objectbank.*;
import edu.stanford.nlp.trees.*;

/**
 * Transforms a Document of Words into a Document of Sentences by grouping the
 * Words. 
 *
 * @author Teg Grenager (grenager@stanford.edu)
 */
public class SentenceRetokenizingProcessor {

  /** The tokenizer to use for retokenizing sentences. */
  private TokenizerFactory tokenizerFactory;

  /**
   * Returns a new <code>List</code> where each element is a Sentence from the
   * <code>List</code> input,
   * retokenized with the tokenizer provided at construction.  Input must be 
   * of class <code>List</code>. Specifically, concatenates the old
   * Sentences with whitespace, and then applies the tokenizer anew.
   */
  public List process(List input) {

    List output = new ArrayList();

    Sentence oldS = null;
    for (Iterator iter=input.iterator(); iter.hasNext(); ) {
      oldS = (Sentence) iter.next();
      output.add(retokenize(oldS));
    }

    return output;


  }

  public Sentence retokenize(Sentence oldS) {
    String sString = oldS.toString();
    Tokenizer toke = tokenizerFactory.getTokenizer(new StringReader(sString));
    Sentence newS = new Sentence(toke.tokenize());
    return newS;
  }


  /** Create a <code>SentenceRetokenizingProcessor</code> that uses the
   *  <code>TokenizerFactory</code> tokenizerFactory.
   */
  public SentenceRetokenizingProcessor(TokenizerFactory tokenizerFactory) {
    this.tokenizerFactory = tokenizerFactory;
  }

}
