package edu.stanford.nlp.process;

import edu.stanford.nlp.util.Filter;

/**
 * Filter that first transforms input before filtering it. An object is
 * accepted iff the given filter accepts its output from the given Function.
 * This is particularly useful when filtering a list of objects, since for
 * example the Function might return a single field or method result from
 * the objects being filtered, and thus the Object could be filtered based
 * on some part of it without having to write a special filter.
 *  @author Christopher Manning
 */
public class TransformedFilter {
    /**
     * Filter that first transforms input before filtering it. An object is
     * accepted iff the given filter accepts its output from the given Function.
     * This is particularly useful when filtering a list of objects, since for
     * example the Function might return a single field or method result from
     * the objects being filtered, and thus the Object could be filtered based
     * on some part of it without having to write a special filter.
     */
    public static Filter filter(Filter filter,Function appliable)
    {
        return(new TransformedFilterImpl(filter,appliable));
    }

    /** Filters on output of Function, returns matching input. */
    private static class TransformedFilterImpl implements Filter
    {
        private Filter filter;
        private Function appliable;

        public TransformedFilterImpl(Filter filter,Function appliable)
        {
            this.filter=filter;
            this.appliable=appliable;
        }

        /** Returns o iff filter accepts appliable's transformation of o. */
        public boolean accept(Object o)
        {
            return(filter.accept(appliable.apply(o)));
        }
    }
}
