package edu.stanford.nlp.util;

import java.util.List;
import java.util.AbstractList;


/** A CircleList wraps another list, presenting an apparently infinite
 *  list by wrapping around all requests for an item, as if the list were
 *  circular.  Note that <code>size()</code> returns the true size, but 
 *  <code>get()</code> works for any number.
 * 
 *  @author Dan Klein
 *  @author Christopher Manning
 */
public class CircleList extends AbstractList {

    private final List l;

    public int size() {
	return l.size();
    }

    public Object get(int i) {
	// int ind = (i%size()+size())%size();
	int sz = size();
	int ind = i % sz;
	if (ind < 0) {
	    ind += sz;
	}
	return l.get(ind);
    }

    public CircleList(List l) {
	this.l = l;
    }

}
