package edu.stanford.nlp.util;

import edu.stanford.nlp.util.MapFactory;
import edu.stanford.nlp.util.CollectionFactory;

import java.util.*;

/**
 * User: Teg Grenager
 * Date: Jan 12, 2004
 * Time: 9:15:53 AM
 */
public class CollectionValuedMap implements Map {

  private Map map;
  protected CollectionFactory cf;
  private boolean treatCollectionsAsImmutable;
  protected MapFactory mf;

  public Object put(Object key, Object value) {
    throw new UnsupportedOperationException();
  }

  public void putAll(Map m) {
    throw new UnsupportedOperationException();
  }

  /**
   *
   * @param key
   * @return the Collection mapped to by key.
   */
  public Object get(Object key) {
    Collection c = (Collection) map.get(key);
    if (c == null)
      c = cf.newEmptyCollection();
    return c;
  }

  /**
   * Adds the value to the Collection mapped to by the key.
   * @param key
   * @param value
   */
  public void add(Object key, Object value) {
    if (treatCollectionsAsImmutable) {
      Collection newC = cf.newCollection();
      Collection c = (Collection) map.get(key);
      if (c != null) {
        newC.addAll(c);
      }
      newC.add(value);
      map.put(key, newC); // replacing the old collection
    } else {
      Collection c = (Collection) map.get(key);
      if (c == null) {
        c = cf.newCollection();
        map.put(key, c);
      }
      c.add(value); // modifying to old collection
    }
  }

  /**
   * Adds all of the mappings in m to this CollectionValuedMap.
   * If m is a CollectionValuedMap, it will behave strangely. Use the constructor instead.
   * @param m
   */
  public void addAll(Map m) {
    Iterator i = m.entrySet().iterator();
    while (i.hasNext()) {
      Map.Entry e = (Map.Entry) i.next();
      add(e.getKey(), e.getValue());
    }
  }

  public Object remove(Object key) {
    return map.remove(key);
  }

  public void removeMapping(Object key, Object value) {
    if (treatCollectionsAsImmutable) {
      Collection c = (Collection) map.get(key);
      if (c != null) {
        Collection newC = cf.newCollection();
        newC.addAll(c);
        newC.remove(value);
        map.put(key, newC);
      }

    } else {
      Collection c = (Collection) get(key);
      c.remove(value);
    }
  }

  public void clear() {
    map.clear();
  }

  public boolean containsKey(Object key) {
    return map.containsKey(key);
  }

  public boolean containsValue(Object value) {
    throw new UnsupportedOperationException();
  }

  public boolean isEmpty() {
    return map.isEmpty();
  }

  /**
   * @return a set view of the mappings contained in this map.
   */
  public Set entrySet() {
    return map.entrySet();
  }

  public Set keySet() {
    return map.keySet();
  }

  public int size() {
    return map.size();
  }

  public Collection values() {
    throw new UnsupportedOperationException();
  }

  public boolean equals(Object o) {
    if (this == o) return true;
    if (!(o instanceof CollectionValuedMap)) return false;

    CollectionValuedMap other = (CollectionValuedMap) o;

    if (other.size() != size())
      return false;

    try {
      Iterator i = entrySet().iterator();
      while (i.hasNext()) {
        Map.Entry e = (Map.Entry) i.next();
        Object key = e.getKey();
        Object value = e.getValue();
        if (value == null) {
          if (!(other.get(key) == null && other.containsKey(key)))
            return false;
        } else {
          if (!value.equals(other.get(key)))
            return false;
        }
      }
    } catch (ClassCastException unused) {
      return false;
    } catch (NullPointerException unused) {
      return false;
    }

    return true;
  }

  public int hashCode() {
    return map.hashCode();
  }

  public Object deltaClone() {
    CollectionValuedMap result = new CollectionValuedMap(null, cf, true);
    result.map = new DeltaMap(this.map);
    return result;
  }

  public Object clone() {
    CollectionValuedMap result = new CollectionValuedMap(this);
    return result;
  }

  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("{");

    Iterator i = entrySet().iterator();
    boolean hasNext = i.hasNext();
    while (hasNext) {
      Map.Entry e = (Map.Entry) (i.next());
      Object key = e.getKey();
      Object value = e.getValue();
      buf.append((key == this ? "(this Map)" : key) + "=" +
              (value == this ? "(this Map)" : value));

      hasNext = i.hasNext();
      if (hasNext)
        buf.append(", ");
    }

    buf.append("}");
    return buf.toString();
  }

  public CollectionValuedMap(MapFactory mf, CollectionFactory cf, boolean treatCollectionsAsImmutable) {
    this.mf = mf;
    this.cf = cf;
    this.treatCollectionsAsImmutable = treatCollectionsAsImmutable;
    if (mf != null)
      map = mf.newMap();
  }

  public CollectionValuedMap(CollectionValuedMap cvm) {
    this.mf = cvm.mf;
    this.cf = cvm.cf;
    this.treatCollectionsAsImmutable = cvm.treatCollectionsAsImmutable;
    map = mf.newMap();
    for (Iterator keyI = cvm.map.entrySet().iterator(); keyI.hasNext();) {
      Map.Entry entry = (Map.Entry) keyI.next();
      Object key = entry.getKey();
      Collection c = (Collection) entry.getValue();
      for (Iterator valueI = c.iterator(); valueI.hasNext();) {
        Object value = valueI.next();
        add(key, value);
      }
    }
  }


  public CollectionValuedMap() {
    this(MapFactory.HASH_MAP_FACTORY, CollectionFactory.HASH_SET_FACTORY, false);
  }

  /**
   * For testing only.
   * @param args from command line
   */
  public static void main(String[] args) {
    CollectionValuedMap originalMap = new CollectionValuedMap();
/*
    for (int i=0; i<4; i++) {
      for (int j=0; j<4; j++) {
        originalMap.add(new Integer(i), new Integer(j));
      }
    }
    originalMap.remove(new Integer(2));
    System.out.println("Map: ");
    System.out.println(originalMap);
    System.exit(0);
*/
    Random r = new Random();
    for (int i = 0; i < 800; i++) {
      Integer rInt1 = new Integer(r.nextInt(400));
      Integer rInt2 = new Integer(r.nextInt(400));
      originalMap.add(rInt1, rInt2);
      System.out.println("Adding " + rInt1 + " " + rInt2);
    }
    CollectionValuedMap originalCopyMap = new CollectionValuedMap(originalMap);
    CollectionValuedMap deltaCopyMap = new CollectionValuedMap(originalMap);
    CollectionValuedMap deltaMap = new DeltaCollectionValuedMap(originalMap);
    // now make a lot of changes to deltaMap;
    // add and change some stuff
    for (int i = 0; i < 400; i++) {
      Integer rInt1 = new Integer(r.nextInt(400));
      Integer rInt2 = new Integer(r.nextInt(400)+1000);
      deltaMap.add(rInt1, rInt2);
      deltaCopyMap.add(rInt1, rInt2);
      System.out.println("Adding " + rInt1 + " " + rInt2);
    }
    // remove some stuff
    for (int i = 0; i < 400; i++) {
      Integer rInt1 = new Integer(r.nextInt(1400));
      Integer rInt2 = new Integer(r.nextInt(1400));
      deltaMap.removeMapping(rInt1, rInt2);
      deltaCopyMap.removeMapping(rInt1, rInt2);
      System.out.println("Removing " + rInt1 + " " + rInt2);
    }
    System.out.println("original: " + originalMap);
    System.out.println("copy: " + deltaCopyMap);
    System.out.println("delta: " + deltaMap);

    System.out.println("Original preserved? " + originalCopyMap.equals(originalMap));
    System.out.println("Delta accurate? " + deltaMap.equals(deltaCopyMap));
  }

}
