package edu.stanford.nlp.util;

import java.util.*;
import java.io.Serializable;

/**
 * A map which contains a pointer to another map, and refers all
 * operations to the second map.  The purpose of this class is to
 * provide a parent to classes (like Counter) which extend Map
 * functionality but wish to allow any backing map implementation.
 *
 * @author Dan Klein
 */
public class CompositionMap implements Map, Serializable {

  private Map map;

  public Map getMap() {
    return map;
  }

  public Map setMap(Map map) {
    Map oldMap = this.map;
    this.map = map;
    return oldMap;
  }

  public CompositionMap() {
    this(new HashMap());
  }

  public CompositionMap(Map map) {
    setMap(map);
  }

  // The rest of the code is to forward operations to the backing map

  public int size() { return map.size(); }
  public boolean isEmpty() { return map.isEmpty(); }
  public boolean containsValue(Object value) { return map.containsValue(value); }
  public boolean containsKey(Object key) { return map.containsKey(key); }
  public Object get(Object key) { return map.get(key); }
  public Object put(Object key, Object value) { return map.put(key,value); }
  public Object remove(Object key) { return map.remove(key); }
  public void putAll(Map t) { map.putAll(t); }
  public void clear() { map.clear(); }
  public Set keySet() { return map.keySet(); }
  public Collection values() { return map.values(); }
  public Set entrySet() { return map.entrySet(); }
  public boolean equals(Object o) { return map.equals(o); }
  public int hashCode() { return map.hashCode(); }
  public String toString() { return map.toString(); }

  public static void main(String[] args) throws Exception {
    Map map = new CompositionMap();
    for (int i=0; i<1000000; i++) {
      Integer o = new Integer(i);
      map.put(o,o);
    }
  }
}
