package edu.stanford.nlp.util;

import java.util.*;

/**
 * @author Dan Klein
 */
public class EditDistance {
    double[][] score = null;

    void clear(int sourceLength, int targetLength) {
        if (score == null || score.length < sourceLength + 1 || score[0].length
                < targetLength + 1) {
            score = new double[sourceLength + 1][targetLength + 1];
        }
        for (int i = 0; i < score.length; i++)
            Arrays.fill(score[i], worst());
    }

    // CONSTRAINT SEMIRING START

    double best() {
        return 0.0;
    }

    double worst() {
        return Double.POSITIVE_INFINITY;
    }

    double unit() {
        return 1.0;
    }

    double better(double x, double y) {
        if (x < y)
            return x;
        return y;
    }

    double combine(double x, double y) {
        return x + y;
    }

    // CONSTRAINT SEMIRING END

    // COST FUNCTION BEGIN

    double insertCost(Object o) {
        return unit();
    }

    double deleteCost(Object o) {
        return unit();
    }

    double substituteCost(Object source, Object target) {
        if (source.equals(target))
            return best();
        return unit();
    }

    double transposeCost(Object s1, Object s2, Object t1, Object t2) {
        if (s1.equals(t2) && s2.equals(t1))
            return unit();
        return worst();
    }

    // COST FUNCTION END

    double score(Object[] source, int sPos, Object[] target, int tPos) {
        double bscore = score[sPos][tPos];
        if (bscore == worst()) {
            if (sPos == 0 && tPos == 0) {
                bscore = best();
            } else {
                if (sPos > 0) {
                    bscore = better(bscore, (combine(score(source, sPos - 1, target,
                            tPos),
                            deleteCost(source[sPos - 1]))));
                }
                if (tPos > 0) {
                    bscore = better(bscore, (combine(score(source, sPos, target,
                            tPos - 1),
                            insertCost(target[tPos - 1]))));
                }
                if (sPos > 0 && tPos > 0) {
                    bscore = better(bscore, (combine(score(source, sPos - 1, target,
                            tPos - 1),
                            substituteCost(source[sPos - 1],
                                    target[tPos - 1]))));
                }
                if (sPos > 1 && tPos > 1) {
                    bscore = better(bscore, (combine(score(source, sPos - 2, target,
                            tPos - 2),
                            transposeCost(source[sPos - 2],
                                    source[sPos - 1], target[tPos - 2], target[tPos - 1]))));
                }
            }
            score[sPos][tPos] = bscore;
        }
        return bscore;
    }

    public double score(Object[] source, Object[] target) {
        clear(source.length, target.length);
        return score(source, source.length, target, target.length);
    }

    public static void main(String[] args) {
        EditDistance d = new EditDistance();
        System.out.println(d.score(Characters.asCharacterArray(args[0]),
                Characters.asCharacterArray(args[1])));
    }

}
