package edu.stanford.nlp.util;

import java.util.*;

/**
 * Contains convenience methods, and utilities for java.util.Iterators
 *
 * @author Huy Nguyen (htnguyen@cs.stanford.edu)
 */
public class Iterators {
    /**
     * Creates an ArrayList containing all of the Objects returned by the given Iterator.
     */
    public static ArrayList asArrayList(Iterator iter) {
        ArrayList al = new ArrayList();
        return (ArrayList) addAll(iter, al);
    }

    /**
     * Creates a HashSet containing all of the Objects returned by the given Iterator.
     */
    public static HashSet asHashSet(Iterator iter) {
        HashSet hs = new HashSet();
        return (HashSet) addAll(iter, hs);
    }

    /**
     * Creates a new Collection from the given CollectionFactory, and adds all of the Objects
     * returned by the given Iterator.
     */
    public static Collection asCollection(Iterator iter, CollectionFactory cf) {
        Collection c = cf.newCollection();
        return addAll(iter, c);

    }

    /**
     * Adds all of the Objects returned by the given Iterator into the given Collection.
     * @return the given Collection
     */
    public static Collection addAll(Iterator iter, Collection c) {
        while (iter.hasNext())
            c.add(iter.next());
        return c;
    }

    /** For internal debugging purposes only. */
    public static void main(String[] args) {
        String[] test = { "a", "b", "c" };

        List l = Arrays.asList(test);

        System.out.println(asArrayList(l.iterator()));

        System.out.println(asHashSet(l.iterator()));

        System.out.println(asCollection(l.iterator(), CollectionFactory.HASH_SET_FACTORY));

        ArrayList al = new ArrayList();
        al.add("d");
        System.out.println(addAll(l.iterator(), al));
    }
}
