package edu.stanford.nlp.util;

import java.util.*;
import java.io.Serializable;

/**
 * Factory for vending Maps.
 *
 * @author Dan Klein (klein@cs.stanford.edu)
 */
abstract public class MapFactory implements Serializable {
  public static final MapFactory HASH_MAP_FACTORY = new HashMapFactory();
  public static final MapFactory IDENTITY_HASH_MAP_FACTORY = new IdentityHashMapFactory();
  public static final MapFactory WEAK_HASH_MAP_FACTORY = new WeakHashMapFactory();
  public static final MapFactory TREE_MAP_FACTORY = new TreeMapFactory();

  public static class HashMapFactory extends MapFactory {
    public Map newMap() { return new HashMap(); }
  }
  public static class IdentityHashMapFactory extends MapFactory {
    public Map newMap() { return new IdentityHashMap(); }
  }
  public static class WeakHashMapFactory extends MapFactory {
    public Map newMap() { return new WeakHashMap(); }
  }
  public static class TreeMapFactory extends MapFactory {
    public Map newMap() { return new TreeMap(); }
  }

  abstract public Map newMap();
}
