package edu.stanford.nlp.util;

import java.util.List;
import java.util.AbstractList;


/** A PaddedList wraps another list, presenting an apparently infinite
 *  list by padding outside the real confines of the list with a default
 *  value.  Note that <code>size()</code> returns the true size, but 
 *  <code>get()</code> works for any number.
 * 
 *  @author Christopher Manning
 */
public class PaddedList extends AbstractList {

    private final List l;
    private final Object padding;

    public int size() {
	return l.size();
    }

    public Object get(int i) {
	if (i < 0 || i >= size()) {
	    return padding;
	} else {
	    return l.get(i);
	}
    }

    public PaddedList(List l) {
	this(l, null);
    }

    public PaddedList(List l, Object padding) {
	this.l = l;
	this.padding = padding;
    }

}
