package edu.stanford.nlp.util;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: Teg Grenager
 * Date: Dec 11, 2003
 * Time: 11:31:51 AM
 * To change this template use Options | File Templates.
 */
public interface PriorityQueue extends Set {

  /**
   * Finds the object with the highest priority, removes it,
   * and returns it.
   *
   * @return the object with highest priority
   */
  public Object removeFirst();

  /**
   * Finds the object with the highest priority and returns it, without
   * modifying the queue.
   *
   * @return the object with minimum key
   */
  public Object getFirst();

  /**
   * Get the priority of a key.
   * @param key The object to assess
   * @return A key's priority. If the key is not in the queue, 
   *      Double.NEGATIVE_INFINITY is returned.
   */
  public double getPriority(Object key);

  /**
   * Convenience method for if you want to pretend increaseKey doesn't exist,
   * or if you really want add's return conditions.
   *
   * @return <tt>true</tt> if this set did not already contain the specified
   *         element.
   */
  public boolean add(Object key, double priority);


  /**
   * Changes a priority, either up or down, adding the key it if it wasn't there already.
   *
   * @param key an <code>Object</code> value
   * @return whether the priority actually changed.
   */
  public boolean changePriority(Object key, double priority);

  /**
   * Increases the priority of the Object key to the new priority if the old priority
   * was lower than the new priority. Otherwise, does nothing.
   * @param key
   * @param priority
   * @return
   */
  public boolean relaxPriority(Object key, double priority);

  public List toSortedList();

}
