package edu.stanford.nlp.util;

import edu.stanford.nlp.util.Filter;
import edu.stanford.nlp.util.ConcatenationIterator;
import edu.stanford.nlp.util.FilteredIterator;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Iterator over the union of two sets.
 *
 * User: Dan Klein (klein@cs.stanford.edu)
 * Date: Oct 22, 2003
 * Time: 7:52:32 PM
 */
public class UnionIterator extends ConcatenationIterator {
  static class NonContainmentFilter implements Filter {
    Set s = null;

    public boolean accept(Object o) {
      return ! s.contains(o);
    }

    NonContainmentFilter(Set s) {
      this.s = s;
    }
  }

  public UnionIterator(Set x, Set y) {
    super(x.iterator(), new FilteredIterator(y.iterator(), new NonContainmentFilter(x)));
  }

  public static void main(String[] args) {
    Set s1 = new HashSet(Arrays.asList(new String[] {"a", "b", "c"}));
    Set s2 = new HashSet(Arrays.asList(new String[] {"a", "d", "b"}));
    Iterator i = new UnionIterator(s1, s2);
    while (i.hasNext()) {
      System.out.println("Accepted: "+i.next());
    }
  }
}
